/**
    This file is autogenerated
*/

{
  py::class_<PigeonIMU> cls(m, "PigeonIMU");
  cls.def(py::init<>())
    .def("create2", &PigeonIMU::create2, release_gil())
    .def("create1", &PigeonIMU::create1, release_gil())
    .def("setYaw", &PigeonIMU::setYaw, release_gil())
    .def("addYaw", &PigeonIMU::addYaw, release_gil())
    .def("setYawToCompass", &PigeonIMU::setYawToCompass, release_gil())
    .def("setFusedHeading", &PigeonIMU::setFusedHeading, release_gil())
    .def("addFusedHeading", &PigeonIMU::addFusedHeading, release_gil())
    .def("setFusedHeadingToCompass", &PigeonIMU::setFusedHeadingToCompass, release_gil())
    .def("setAccumZAngle", &PigeonIMU::setAccumZAngle, release_gil())
    .def("configTemperatureCompensationEnable", &PigeonIMU::configTemperatureCompensationEnable, release_gil())
    .def("setCompassDeclination", &PigeonIMU::setCompassDeclination, release_gil())
    .def("setCompassAngle", &PigeonIMU::setCompassAngle, release_gil())
    .def("enterCalibrationMode", &PigeonIMU::enterCalibrationMode, release_gil())
    .def("getGeneralStatus", &PigeonIMU::getGeneralStatus, release_gil())
    .def("getLastError", &PigeonIMU::getLastError, release_gil())
    .def("get6dQuaternion", &PigeonIMU::get6dQuaternion, release_gil())
    .def("getYawPitchRoll", &PigeonIMU::getYawPitchRoll, release_gil())
    .def("getAccumGyro", &PigeonIMU::getAccumGyro, release_gil())
    .def("getAbsoluteCompassHeading", &PigeonIMU::getAbsoluteCompassHeading, release_gil())
    .def("getCompassHeading", &PigeonIMU::getCompassHeading, release_gil())
    .def("getCompassFieldStrength", &PigeonIMU::getCompassFieldStrength, release_gil())
    .def("getTemp", &PigeonIMU::getTemp, release_gil())
    .def("getState", &PigeonIMU::getState, release_gil())
    .def("getUpTime", &PigeonIMU::getUpTime, release_gil())
    .def("getRawMagnetometer", &PigeonIMU::getRawMagnetometer, release_gil())
    .def("getBiasedMagnetometer", &PigeonIMU::getBiasedMagnetometer, release_gil())
    .def("getBiasedAccelerometer", &PigeonIMU::getBiasedAccelerometer, release_gil())
    .def("getRawGyro", &PigeonIMU::getRawGyro, release_gil())
    .def("getAccelerometerAngles", &PigeonIMU::getAccelerometerAngles, release_gil())
    .def("getFusedHeading2", &PigeonIMU::getFusedHeading2, release_gil())
    .def("getFusedHeading1", &PigeonIMU::getFusedHeading1, release_gil())
    .def("getResetCount", &PigeonIMU::getResetCount, release_gil())
    .def("getResetFlags", &PigeonIMU::getResetFlags, release_gil())
    .def("getFirmwareVersion", &PigeonIMU::getFirmwareVersion, release_gil())
    .def("hasResetOccurred", &PigeonIMU::hasResetOccurred, release_gil())
    .def("setLastError", &PigeonIMU::setLastError, release_gil())
    .def("getFaults", &PigeonIMU::getFaults, release_gil())
    .def("getStickyFaults", &PigeonIMU::getStickyFaults, release_gil())
    .def("clearStickyFaults", &PigeonIMU::clearStickyFaults, release_gil())
    .def("setStatusFramePeriod", &PigeonIMU::setStatusFramePeriod, release_gil())
    .def("getStatusFramePeriod", &PigeonIMU::getStatusFramePeriod, release_gil())
    .def("setControlFramePeriod", &PigeonIMU::setControlFramePeriod, release_gil())
  ;
}