# validated: 2018-01-06 EN d090f9cdc6a1 libraries/driver/include/ctre/phoenix/CANifierStickyFaults.h
from .faultsbase import FaultsBase

__all__ = ['CANifierStickyFaults']


class CANifierStickyFaults(FaultsBase):
    pass
