/**
    This file is autogenerated
*/

{
  py::class_<{{ module_name }}> cls(m, "{{ module_name }}");
  cls.def(py::init<>())
  {% for fn in header.functions %}
  {% if 'ignore' not in fn.data %}
    .def("{{ 'None_' if fn.x_name == 'None' else fn.x_name }}", &{{ module_name }}::{{ fn.x_name }}, release_gil())
  {% endif %}
  {% endfor %}
  ;
}
