/**
    This file is autogenerated
*/

{
  py::class_<CANifier> cls(m, "CANifier");
  cls.def(py::init<>())
    .def("create1", &CANifier::create1, release_gil())
    .def("setLEDOutput", &CANifier::setLEDOutput, release_gil())
    .def("setGeneralOutputs", &CANifier::setGeneralOutputs, release_gil())
    .def("setGeneralOutput", &CANifier::setGeneralOutput, release_gil())
    .def("setPWMOutput", &CANifier::setPWMOutput, release_gil())
    .def("enablePWMOutput", &CANifier::enablePWMOutput, release_gil())
    .def("getGeneralInput", &CANifier::getGeneralInput, release_gil())
    .def("getPWMInput", &CANifier::getPWMInput, release_gil())
    .def("getLastError", &CANifier::getLastError, release_gil())
    .def("getBusVoltage", &CANifier::getBusVoltage, release_gil())
    .def("setLastError", &CANifier::setLastError, release_gil())
    .def("configSetParameter", &CANifier::configSetParameter, release_gil())
    .def("configGetParameter", &CANifier::configGetParameter, release_gil())
    .def("configSetCustomParam", &CANifier::configSetCustomParam, release_gil())
    .def("configGetCustomParam", &CANifier::configGetCustomParam, release_gil())
    .def("getFaults", &CANifier::getFaults, release_gil())
    .def("getStickyFaults", &CANifier::getStickyFaults, release_gil())
    .def("clearStickyFaults", &CANifier::clearStickyFaults, release_gil())
    .def("getFirmwareVersion", &CANifier::getFirmwareVersion, release_gil())
    .def("hasResetOccurred", &CANifier::hasResetOccurred, release_gil())
    .def("setStatusFramePeriod", &CANifier::setStatusFramePeriod, release_gil())
    .def("getStatusFramePeriod", &CANifier::getStatusFramePeriod, release_gil())
    .def("setControlFramePeriod", &CANifier::setControlFramePeriod, release_gil())
  ;
}